import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import uid from '@temporg/uid';
import generatePropCombinations from './generatePropCombinations';
export default function generateComponentExamples(Component) {
  var config = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {
    sectionProp: null,
    propValues: {},
    maxExamplesPerPage: null,
    excludeProps: [],
    getExampleProps: function getExampleProps(props, index) {
      return {};
    },
    getComponentProps: function getComponentProps(props, index) {
      return {};
    },
    getParameters: function getParameters(examples, pageIndex) {
      return {};
    },
    filter: function filter(props, index) {
      return false;
    }
  };
  var sectionProp = config.sectionProp,
      excludeProps = config.excludeProps,
      filter = config.filter;
  var PROPS_CACHE = [];
  var sections = [];
  var maxExamples = config.maxExamples || 500;
  var exampleCount = 0;
  var propValues = {};

  var getParameters = function getParameters(_ref) {
    var examples = _ref.examples,
        index = _ref.index;
    var parameters = {};

    if (typeof config.getParameters === 'function') {
      parameters = _objectSpread({}, config.getParameters(examples, index));
    }

    return parameters;
  };

  var getComponentProps = function getComponentProps(props) {
    var componentProps = props;

    if (typeof config.getComponentProps === 'function') {
      componentProps = _objectSpread({}, componentProps, {}, config.getComponentProps(props));
    }

    return componentProps;
  };

  var getExampleProps = function getExampleProps(props) {
    var exampleProps = {};

    if (typeof config.getExampleProps === 'function') {
      exampleProps = _objectSpread({}, config.getExampleProps(props));
    }

    return exampleProps;
  };

  var addPage = function addPage(section) {
    var page = {
      examples: [],
      index: section.pages.length
    };
    section.pages.push(page);
    return page;
  };

  var addExample = function addExample() {
    var sectionName = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : 'Examples';
    var example = arguments.length > 1 ? arguments[1] : void 0;
    var section = sections.find(function (section) {
      return section.sectionName === sectionName;
    });

    if (!section) {
      section = {
        sectionName: sectionName,
        propName: sectionProp,
        propValue: sectionName,
        pages: []
      };
      sections.push(section);
    }

    var page = section.pages[section.pages.length - 1];
    var maxExamplesPerPage = config.maxExamplesPerPage;

    if (typeof maxExamplesPerPage === 'function') {
      maxExamplesPerPage = maxExamplesPerPage(sectionName);
    }

    if (!page) {
      page = addPage(section);
    } else if (maxExamplesPerPage && page.examples.length % maxExamplesPerPage === 0 && page.examples.length > 0) {
      page = addPage(section);
    }

    page.examples.push(example);
  };

  var maybeAddExample = function maybeAddExample(props) {
    var componentProps = getComponentProps(props);
    var exampleProps = getExampleProps(props);
    var key = uid();
    var propsString = JSON.stringify(componentProps);
    var ignore = typeof filter === 'function' ? filter(componentProps) : false;

    if (!ignore && !PROPS_CACHE.includes(propsString)) {
      exampleCount++;

      if (exampleCount < maxExamples) {
        PROPS_CACHE.push(propsString);
        addExample(componentProps[sectionProp], {
          Component: Component,
          componentProps: componentProps,
          exampleProps: exampleProps,
          key: key
        });
      }
    }
  };

  if (isEmpty(config.propValues)) {
    maybeAddExample(config.propValues);
  } else {
    if (Array.isArray(excludeProps)) {
      Object.keys(config.propValues).forEach(function (propName) {
        if (!excludeProps.includes(propName)) {
          propValues[propName] = config.propValues[propName];
        }
      });
    } else {
      propValues = config.propValues;
    } // eslint-disable-next-line no-console


    console.info("Generating examples for ".concat(Component.displayName, " (").concat(Object.keys(propValues).length, " props)..."));
    var combos = generatePropCombinations(propValues).filter(Boolean);
    var index = 0;

    while (index < combos.length && exampleCount < maxExamples) {
      var combo = combos[index];

      if (combo) {
        maybeAddExample(combo);
        index++;
      }
    }
  }

  if (exampleCount >= maxExamples) {
    console.error("Too many examples for ".concat(Component.displayName, "! Add a filter to the config."));
  } // eslint-disable-next-line no-console


  console.info("Generated ".concat(exampleCount, " examples for ").concat(Component.displayName));
  sections.forEach(function (_ref2) {
    var pages = _ref2.pages;
    pages.forEach(function (page, index) {
      // eslint-disable-next-line no-param-reassign
      page.parameters = getParameters(page);
    });
  });
  return sections;
}

function isEmpty(obj) {
  if (typeof obj !== 'object') return true;

  for (var key in obj) {
    if (hasOwnProperty.call(obj, key)) return false;
  }

  return true;
}