"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generatePropCombinations;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

/**
 * Given possible values for each prop, returns all combinations of those prop values.
 * To generate the prop names and values from the component source see the `parsePropValues` utility
 * and the `component-examples-loader`
 *
 * @param {Object} propValues an object with the shape {propName: arrayOfPossibleValues}
 * @returns {Array} an array of all prop combinations [{propAName: propAValue, propBName: propBValue}]
 */
function generatePropCombinations() {
  var propValues = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : {};
  var propNames = Object.keys(propValues);
  var combos = [];
  if (!propNames.length) return combos;
  var numProps = propNames.length;

  for (var i = 0; i < numProps; i++) {
    var propName = propNames[i];
    var valuesForProp = propValues[propName];

    if (!Array.isArray(valuesForProp) || !valuesForProp.length) {
      throw new Error("[ui-component-examples] Please provide a non-empty array of possible values for prop ".concat(propName));
    }

    var numValues = valuesForProp.length;
    var numCombos = combos.length;

    for (var j = 0; j < numValues; j++) {
      var propValue = valuesForProp[j];

      if (numCombos > 0) {
        for (var k = 0; k < numCombos; k++) {
          var combo = combos[k]; // Check against the keys of the object here. `combo[propName]` could
          // evaluate to a boolean value which will mess up this logic.

          if (!Object.keys(combo).includes(propName)) {
            // eslint-disable-next-line no-param-reassign
            combo[propName] = propValue;
          } else {
            combos.push((0, _objectSpread3.default)({}, combo, (0, _defineProperty2.default)({}, propName, propValue)));
          }
        }
      } else {
        combos.push((0, _defineProperty2.default)({}, propName, propValue));
      }
    }
  }

  return combos;
}