
import path from 'path'
import fs from 'fs'
import loaderUtils from 'loader-utils'
import parsePropValues from './parsePropValues'
import loadConfig from '@temporg/config-loader'

export default function componentExamplesLoader (source, map, meta) {
  this.cacheable && this.cacheable()

  const loader = this
  const callback = loader.async()

  const config = {
    maxExamples: 500,
    ...loadConfig('examples', {})
  }

  const generateComponentExamples = require.resolve('./generateComponentExamples')
  const renderPage = require.resolve('./renderPage')
  const renderExample = require.resolve('./renderExample')
  const configPath = `!!${loaderUtils.getRemainingRequest(loader)}`

  const getComponentPath = (typeof config.getComponentPath === 'function') ?
    config.getComponentPath :
    (configFilePath) => {
      const basePath = path.dirname(configFilePath)
      // try to determine component source based on where the .examples file is:
      return path.resolve(basePath, configFilePath.includes('__examples__') ? '../index.js' : './index.js')
    }

  const componentPath = getComponentPath(this.resourcePath)

  fs.readFile(`${componentPath}${!componentPath.includes('.') ? '.js' : ''}`, 'utf8', (err, componentSrc) => {
    // if (err) return callback(err)
    err && loader.emitWarning(err)

    let generatedPropValues = {}

    if (!err) {
      loader.addDependency(componentPath)
      try {
        generatedPropValues = parsePropValues(componentSrc)
      } catch (error) {
        loader.emitWarning(error)
      }
    }

    const result = `
const generateComponentExamples = require(${JSON.stringify(generateComponentExamples)}).default
const renderPage = require(${JSON.stringify(renderPage)}).default
const renderExample = require(${JSON.stringify(renderExample)}).default
const config = require(${JSON.stringify(configPath)}).default

// merge in generated prop values:
config.propValues = Object.assign(
  ${JSON.stringify(generatedPropValues)},
  config.propValues || {}
)
const Component = require(${JSON.stringify(componentPath)}).default

config.maxExamples = Boolean(config.maxExamples) ? config.maxExamples : ${config.maxExamples}

module.exports = {
 componentName: Component.displayName || Component.name,
 sections: generateComponentExamples(Component, config),
 renderPage: config.renderPage || renderPage,
 renderExample: config.renderExample || renderExample
}
`
    return callback(null, result, map)
  })
}
