
/**
 * Given possible values for each prop, returns all combinations of those prop values.
 * To generate the prop names and values from the component source see the `parsePropValues` utility
 * and the `component-examples-loader`
 *
 * @param {Object} propValues an object with the shape {propName: arrayOfPossibleValues}
 * @returns {Array} an array of all prop combinations [{propAName: propAValue, propBName: propBValue}]
 */
export default function generatePropCombinations (propValues = {}) {
  const propNames = Object.keys(propValues)
  let combos = []

  if (!propNames.length) return combos

  const numProps = propNames.length
  for (let i = 0; i < numProps; i++) {
    const propName = propNames[i]
    const valuesForProp = propValues[propName]

    if (!Array.isArray(valuesForProp) || !valuesForProp.length) {
      throw new Error(`[ui-component-examples] Please provide a non-empty array of possible values for prop ${propName}`)
    }

    const numValues = valuesForProp.length
    const numCombos = combos.length

    for (let j = 0; j < numValues; j++) {
      const propValue = valuesForProp[j]
      if (numCombos > 0) {
        for (let k = 0; k < numCombos; k++) {
          let combo = combos[k]

          // Check against the keys of the object here. `combo[propName]` could
          // evaluate to a boolean value which will mess up this logic.
          if (!Object.keys(combo).includes(propName)) {
            // eslint-disable-next-line no-param-reassign
            combo[propName] = propValue
          } else {
            combos.push({ ...combo, [propName]: propValue })
          }
        }
      } else {
        combos.push({ [propName]: propValue })
      }
    }
  }
  return combos
}
